# Config Distro

Built on the same architecture as Configuration Split, Config Distro
provides a framework for managing configuration updates from distributions.

For a full description of the module, visit the
[project page](https://www.drupal.org/project/config_distro).

Submit bug reports and feature suggestions, or track changes in the
[issue queue](https://www.drupal.org/project/issues/config_distro).


## Table of contents

- Requirements
- Installation
- Configuration
- Maintainers


## Requirements

This module requires the following modules:

[Config Filter](https://www.drupal.org/project/config_filter)


## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).


## Configuration

- Install Config Distro.
- Install one or more modules that provide Config Filter plugins that
  integrate with this module, such as Configuration Synchronizer.
- Update your installed modules or themes to a new versions that include
  configuration changes or updates. Run database updates and clear caches.
- If using the UI, navigate to Configuration > Development > Distribution Update
  (`/admin/config/development/distro`). Review and run updates.
- If using Drush, use the Drush command config-distro-update to run imports.


## Usage

The `config_distro` module provides tools for applying configuration updates from extensions (modules, themes, or install profiles) to the active configuration of a site. It supports multiple modes to control how updates are applied:

### Update Modes

> 💡 The default update mode is configurable at the site level and will be used unless explicitly overridden.
> 💡 For safety, it is a good idea to always create a backup of your site before running `cd-update` commands or importing config via the user interface.

When applying configuration updates from an extension, you can choose from three modes:

- **Merge (mode 1 - default):**  
  Applies new keys and values from the extension config, but preserves any existing overrides made on the site.  
  Useful for non-destructive updates where site-specific customizations should be preserved.

- **Partial Reset (mode 2):**  
  Resets only the keys that exist in the extension’s config.  
  Useful when you want to discard changes to specific config items while retaining custom keys added on the site.

- **Full Reset (mode 3):**  
  Completely replaces the active configuration with the version provided by the extension.  
  This mode is destructive and should only be used when you want to reset configuration to the version defined in code.

Install profiles can be updated using `config_distro` because profiles are treated as extensions in the Drupal system. 
This allows profile maintainers to ship configuration updates after installation.

---

### Using Drush

You can apply configuration updates using :

```bash
drush cd-update
```


Install profiles can be updated using `config_distro` because profiles are treated as extensions in the Drupal 
system. This allows profile maintainers to ship configuration updates after installation.


This command compares the active configuration with configuration provided by extensions. If changes are detected, you'll be shown a preview and asked to confirm before updates are applied.

**Examples:**

- Apply updates using the default mode defined in site configuration:

```
drush cd-update
```

- Override the default and apply updates using partial reset:

```
drush cd-update --update-mode=2
```

- Apply updates using full reset:

```
drush cd-update --update-mode=3
```

- Show a unified diff of the changes before applying:

```
drush cd-update --update-mode=1 --preview=diff
```

---

### Using the UI

Navigate to:  
**Configuration → Development → Distribution Update**  
(`/admin/config/development/distro`)

This interface displays all extensions (modules, themes, or install profiles) that provide configuration. For each, you can:

- View pending changes
- Choose an update mode (merge, partial reset, full reset)
- Preview differences (diff or list view)
- Apply updates interactively

The selected update mode will default to the site's configured setting, but you can override it in the UI for one-off updates.
Be careful not to leave your site in full reset mode unless you plan on using it every time.

## Maintainers

- Antonio De Marco - [ademarco](https://www.drupal.org/u/ademarco)
- Fabian Bircher - [bircher](https://www.drupal.org/u/bircher)
- Joe Parsons - [joegraduate](https://www.drupal.org/u/joegraduate)
- Andrea Pescetti - [pescetti](https://www.drupal.org/u/pescetti)
- Pieter Frenssen - [pfrenssen](https://www.drupal.org/u/pfrenssen)
